/**
 * @file Cone.h
This file has been autogenerated from the official plotly.js source.

If you like Plotly, please support them: L<https://plot.ly/> 
Open source announcement: L<https://plot.ly/javascript/open-source-announcement/>

Full reference: L<https://plot.ly/javascript/reference/#cone>

This is an unofficial Plotly Perl module. Currently I'm not affiliated in any way with Plotly. 
But I think plotly.js is a great library and I want to use it with C++.
 *
 */
#include <vector>
#include <string>
#include <sstream>

#include <json11.hpp>

#include "../BaseTrace.h"

#include "CppPlotly/Trace/Cone/Colorbar.h"
#include "CppPlotly/Trace/Cone/Hoverlabel.h"
#include "CppPlotly/Trace/Cone/Lighting.h"
#include "CppPlotly/Trace/Cone/Lightposition.h"
#include "CppPlotly/Trace/Cone/Stream.h"


namespace CppPlotly {
    namespace Trace {
        /**
         * Use cone traces to visualize vector fields.  Specify a vector field using 6 1D arrays, 3 position arrays `x`, `y` and `z` and 3 vector component arrays `u`, `v`, `w`. The cones are drawn exactly at the positions given by `x`, `y` and `z`.
         *
         *
         */ 
        class Cone : public virtual CppPlotly::BaseTrace {
        public:
            Cone() :
                            _cone({
                                {"type", "cone"},
                                    })
                        {}

           /**
Sets the cones' anchor with respect to their x/y/z positions. Note that *cm* denote the cone's center of mass which corresponds to 1/4 from the tail to tip.
*/Cone & Anchor(const json11::Json &anchor ) {
    _cone.insert({"anchor", anchor});
    return *this;
}


/**
Determines whether the colorscale is a default palette (`autocolorscale: true`) or the palette determined by `colorscale`. In case `colorscale` is unspecified or `autocolorscale` is true, the default  palette will be chosen according to whether numbers in the `color` array are all positive, all negative or mixed.
*/Cone & Autocolorscale(const bool &autocolorscale ) {
    _cone.insert({"autocolorscale", autocolorscale});
    return *this;
}


/**
Determines whether or not the color domain is computed with respect to the input data (here u/v/w norm) or the bounds set in `cmin` and `cmax`  Defaults to `false` when `cmin` and `cmax` are set by the user.
*/Cone & Cauto(const bool &cauto ) {
    _cone.insert({"cauto", cauto});
    return *this;
}


/**
Sets the upper bound of the color domain. Value should have the same units as u/v/w norm and if set, `cmin` must be set as well.
*/Cone & Cmax(const double &cmax ) {
    _cone.insert({"cmax", cmax});
    return *this;
}


/**
Sets the mid-point of the color domain by scaling `cmin` and/or `cmax` to be equidistant to this point. Value should have the same units as u/v/w norm. Has no effect when `cauto` is `false`.
*/Cone & Cmid(const double &cmid ) {
    _cone.insert({"cmid", cmid});
    return *this;
}


/**
Sets the lower bound of the color domain. Value should have the same units as u/v/w norm and if set, `cmax` must be set as well.
*/Cone & Cmin(const double &cmin ) {
    _cone.insert({"cmin", cmin});
    return *this;
}


Cone & Colorbar(const CppPlotly::Trace::cone::Colorbar &colorbar ) {
    _cone.insert({"colorbar", colorbar});
    return *this;
}


/**
Sets the colorscale. The colorscale must be an array containing arrays mapping a normalized value to an rgb, rgba, hex, hsl, hsv, or named color string. At minimum, a mapping for the lowest (0) and highest (1) values are required. For example, `[[0, 'rgb(0,0,255)', [1, 'rgb(255,0,0)']]`. To control the bounds of the colorscale in color space, use`cmin` and `cmax`. Alternatively, `colorscale` may be a palette name string of the following list: Greys,YlGnBu,Greens,YlOrRd,Bluered,RdBu,Reds,Blues,Picnic,Rainbow,Portland,Jet,Hot,Blackbody,Earth,Electric,Viridis,Cividis.
*/Cone & Colorscale(const json11::Json &colorscale ) {
    _cone.insert({"colorscale", colorscale});
    return *this;
}


/**
Assigns extra data each datum. This may be useful when listening to hover, click and selection events. Note that, *scatter* traces also appends customdata items in the markers DOM elements
*/Cone & Customdata(const std::vector<double> &customdata ) {
    _cone.insert({"customdata", customdata});
    return *this;
}


/**
Assigns extra data each datum. This may be useful when listening to hover, click and selection events. Note that, *scatter* traces also appends customdata items in the markers DOM elements
*/Cone & Customdata(const std::vector<std::string> &customdata ) {
    _cone.insert({"customdata", customdata});
    return *this;
}


/**
Assigns extra data each datum. This may be useful when listening to hover, click and selection events. Note that, *scatter* traces also appends customdata items in the markers DOM elements
*/Cone & Customdata(const json11::Json &customdata ) {
    _cone.insert({"customdata", customdata});
    return *this;
}


/**
Sets the source reference on plot.ly for  customdata .
*/Cone & Customdatasrc(const std::string &customdatasrc ) {
    _cone.insert({"customdatasrc", customdatasrc});
    return *this;
}


/**
Determines which trace information appear on hover. If `none` or `skip` are set, no information is displayed upon hovering. But, if `none` is set, click and hover events are still fired.
*/Cone & Hoverinfo(const json11::Json &hoverinfo ) {
    _cone.insert({"hoverinfo", hoverinfo});
    return *this;
}


/**
Sets the source reference on plot.ly for  hoverinfo .
*/Cone & Hoverinfosrc(const std::string &hoverinfosrc ) {
    _cone.insert({"hoverinfosrc", hoverinfosrc});
    return *this;
}


Cone & Hoverlabel(const CppPlotly::Trace::cone::Hoverlabel &hoverlabel ) {
    _cone.insert({"hoverlabel", hoverlabel});
    return *this;
}


/**
Template string used for rendering the information that appear on hover box. Note that this will override `hoverinfo`. Variables are inserted using %{variable}, for example "y: %{y}". Numbers are formatted using d3-format's syntax %{variable:d3-format}, for example "Price: %{y:$.2f}". See https://github.com/d3/d3-format/blob/master/README.md#locale_format for details on the formatting syntax. The variables available in `hovertemplate` are the ones emitted as event data described at this link https://plot.ly/javascript/plotlyjs-events/#event-data. Additionally, every attributes that can be specified per-point (the ones that are `arrayOk: true`) are available. variable `norm` Anything contained in tag `<extra>` is displayed in the secondary box, for example "<extra>{fullData.name}</extra>". To hide the secondary box completely, use an empty tag `<extra></extra>`.
*/Cone & Hovertemplate(const json11::Json &hovertemplate ) {
    _cone.insert({"hovertemplate", hovertemplate});
    return *this;
}


/**
Sets the source reference on plot.ly for  hovertemplate .
*/Cone & Hovertemplatesrc(const std::string &hovertemplatesrc ) {
    _cone.insert({"hovertemplatesrc", hovertemplatesrc});
    return *this;
}


/**
Same as `text`.
*/Cone & Hovertext(const json11::Json &hovertext ) {
    _cone.insert({"hovertext", hovertext});
    return *this;
}


/**
Sets the source reference on plot.ly for  hovertext .
*/Cone & Hovertextsrc(const std::string &hovertextsrc ) {
    _cone.insert({"hovertextsrc", hovertextsrc});
    return *this;
}


/**
Assigns id labels to each datum. These ids for object constancy of data points during animation. Should be an array of strings, not numbers or any other type.
*/Cone & Ids(const std::vector<double> &ids ) {
    _cone.insert({"ids", ids});
    return *this;
}


/**
Assigns id labels to each datum. These ids for object constancy of data points during animation. Should be an array of strings, not numbers or any other type.
*/Cone & Ids(const std::vector<std::string> &ids ) {
    _cone.insert({"ids", ids});
    return *this;
}


/**
Assigns id labels to each datum. These ids for object constancy of data points during animation. Should be an array of strings, not numbers or any other type.
*/Cone & Ids(const json11::Json &ids ) {
    _cone.insert({"ids", ids});
    return *this;
}


/**
Sets the source reference on plot.ly for  ids .
*/Cone & Idssrc(const std::string &idssrc ) {
    _cone.insert({"idssrc", idssrc});
    return *this;
}


/**
Sets the legend group for this trace. Traces part of the same legend group hide/show at the same time when toggling legend items.
*/Cone & Legendgroup(const std::string &legendgroup ) {
    _cone.insert({"legendgroup", legendgroup});
    return *this;
}


Cone & Lighting(const CppPlotly::Trace::cone::Lighting &lighting ) {
    _cone.insert({"lighting", lighting});
    return *this;
}


Cone & Lightposition(const CppPlotly::Trace::cone::Lightposition &lightposition ) {
    _cone.insert({"lightposition", lightposition});
    return *this;
}


/**
Sets the trace name. The trace name appear as the legend item and on hover.
*/Cone & Name(const std::string &name ) {
    _cone.insert({"name", name});
    return *this;
}


/**
Sets the opacity of the surface. Please note that in the case of using high `opacity` values for example a value greater than or equal to 0.5 on two surfaces (and 0.25 with four surfaces), an overlay of multiple transparent surfaces may not perfectly be sorted in depth by the webgl API. This behavior may be improved in the near future and is subject to change.
*/Cone & Opacity(const double &opacity ) {
    _cone.insert({"opacity", opacity});
    return *this;
}


/**
Reverses the color mapping if true. If true, `cmin` will correspond to the last color in the array and `cmax` will correspond to the first color.
*/Cone & Reversescale(const bool &reversescale ) {
    _cone.insert({"reversescale", reversescale});
    return *this;
}


/**
Sets a reference between this trace's 3D coordinate system and a 3D scene. If *scene* (the default value), the (x,y,z) coordinates refer to `layout.scene`. If *scene2*, the (x,y,z) coordinates refer to `layout.scene2`, and so on.
*/Cone & Scene(const json11::Json &scene ) {
    _cone.insert({"scene", scene});
    return *this;
}


/**
Array containing integer indices of selected points. Has an effect only for traces that support selections. Note that an empty array means an empty selection where the `unselected` are turned on for all points, whereas, any other non-array values means no selection all where the `selected` and `unselected` styles have no effect.
*/Cone & Selectedpoints(const json11::Json &selectedpoints ) {
    _cone.insert({"selectedpoints", selectedpoints});
    return *this;
}


/**
Determines whether or not an item corresponding to this trace is shown in the legend.
*/Cone & Showlegend(const bool &showlegend ) {
    _cone.insert({"showlegend", showlegend});
    return *this;
}


/**
Determines whether or not a colorbar is displayed for this trace.
*/Cone & Showscale(const bool &showscale ) {
    _cone.insert({"showscale", showscale});
    return *this;
}


/**
Determines whether `sizeref` is set as a *scaled* (i.e unitless) scalar (normalized by the max u/v/w norm in the vector field) or as *absolute* value (in the same units as the vector field).
*/Cone & Sizemode(const json11::Json &sizemode ) {
    _cone.insert({"sizemode", sizemode});
    return *this;
}


/**
Adjusts the cone size scaling. The size of the cones is determined by their u/v/w norm multiplied a factor and `sizeref`. This factor (computed internally) corresponds to the minimum "time" to travel across two successive x/y/z positions at the average velocity of those two successive positions. All cones in a given trace use the same factor. With `sizemode` set to *scaled*, `sizeref` is unitless, its default value is *0.5* With `sizemode` set to *absolute*, `sizeref` has the same units as the u/v/w vector field, its the default value is half the sample's maximum vector norm.
*/Cone & Sizeref(const double &sizeref ) {
    _cone.insert({"sizeref", sizeref});
    return *this;
}


Cone & Stream(const CppPlotly::Trace::cone::Stream &stream ) {
    _cone.insert({"stream", stream});
    return *this;
}


/**
Sets the text elements associated with the cones. If trace `hoverinfo` contains a *text* flag and *hovertext* is not set, these elements will be seen in the hover labels.
*/Cone & Text(const json11::Json &text ) {
    _cone.insert({"text", text});
    return *this;
}


/**
Sets the source reference on plot.ly for  text .
*/Cone & Textsrc(const std::string &textsrc ) {
    _cone.insert({"textsrc", textsrc});
    return *this;
}


/**
Sets the x components of the vector field.
*/Cone & U(const std::vector<double> &u ) {
    _cone.insert({"u", u});
    return *this;
}


/**
Sets the x components of the vector field.
*/Cone & U(const std::vector<std::string> &u ) {
    _cone.insert({"u", u});
    return *this;
}


/**
Sets the x components of the vector field.
*/Cone & U(const json11::Json &u ) {
    _cone.insert({"u", u});
    return *this;
}


/**
Assign an id to this trace, Use this to provide object constancy between traces during animations and transitions.
*/Cone & Uid(const std::string &uid ) {
    _cone.insert({"uid", uid});
    return *this;
}


/**
Controls persistence of some user-driven changes to the trace: `constraintrange` in `parcoords` traces, as well as some `editable: true` modifications such as `name` and `colorbar.title`. Defaults to `layout.uirevision`. Note that other user-driven trace attribute changes are controlled by `layout` attributes: `trace.visible` is controlled by `layout.legend.uirevision`, `selectedpoints` is controlled by `layout.selectionrevision`, and `colorbar.(x|y)` (accessible with `config: {editable: true}`) is controlled by `layout.editrevision`. Trace changes are tracked by `uid`, which only falls back on trace index if no `uid` is provided. So if your app can add/remove traces before the end of the `data` array, such that the same trace has a different index, you can still preserve user-driven changes if you give each trace a `uid` that stays with it as it moves.
*/Cone & Uirevision(const json11::Json &uirevision ) {
    _cone.insert({"uirevision", uirevision});
    return *this;
}


/**
Sets the source reference on plot.ly for  u .
*/Cone & Usrc(const std::string &usrc ) {
    _cone.insert({"usrc", usrc});
    return *this;
}


/**
Sets the y components of the vector field.
*/Cone & V(const std::vector<double> &v ) {
    _cone.insert({"v", v});
    return *this;
}


/**
Sets the y components of the vector field.
*/Cone & V(const std::vector<std::string> &v ) {
    _cone.insert({"v", v});
    return *this;
}


/**
Sets the y components of the vector field.
*/Cone & V(const json11::Json &v ) {
    _cone.insert({"v", v});
    return *this;
}


/**
Determines whether or not this trace is visible. If *legendonly*, the trace is not drawn, but can appear as a legend item (provided that the legend itself is visible).
*/Cone & Visible(const json11::Json &visible ) {
    _cone.insert({"visible", visible});
    return *this;
}


/**
Sets the source reference on plot.ly for  v .
*/Cone & Vsrc(const std::string &vsrc ) {
    _cone.insert({"vsrc", vsrc});
    return *this;
}


/**
Sets the z components of the vector field.
*/Cone & W(const std::vector<double> &w ) {
    _cone.insert({"w", w});
    return *this;
}


/**
Sets the z components of the vector field.
*/Cone & W(const std::vector<std::string> &w ) {
    _cone.insert({"w", w});
    return *this;
}


/**
Sets the z components of the vector field.
*/Cone & W(const json11::Json &w ) {
    _cone.insert({"w", w});
    return *this;
}


/**
Sets the source reference on plot.ly for  w .
*/Cone & Wsrc(const std::string &wsrc ) {
    _cone.insert({"wsrc", wsrc});
    return *this;
}


/**
Sets the x coordinates of the vector field and of the displayed cones.
*/Cone & X(const std::vector<double> &x ) {
    _cone.insert({"x", x});
    return *this;
}


/**
Sets the x coordinates of the vector field and of the displayed cones.
*/Cone & X(const std::vector<std::string> &x ) {
    _cone.insert({"x", x});
    return *this;
}


/**
Sets the x coordinates of the vector field and of the displayed cones.
*/Cone & X(const json11::Json &x ) {
    _cone.insert({"x", x});
    return *this;
}


/**
Sets the source reference on plot.ly for  x .
*/Cone & Xsrc(const std::string &xsrc ) {
    _cone.insert({"xsrc", xsrc});
    return *this;
}


/**
Sets the y coordinates of the vector field and of the displayed cones.
*/Cone & Y(const std::vector<double> &y ) {
    _cone.insert({"y", y});
    return *this;
}


/**
Sets the y coordinates of the vector field and of the displayed cones.
*/Cone & Y(const std::vector<std::string> &y ) {
    _cone.insert({"y", y});
    return *this;
}


/**
Sets the y coordinates of the vector field and of the displayed cones.
*/Cone & Y(const json11::Json &y ) {
    _cone.insert({"y", y});
    return *this;
}


/**
Sets the source reference on plot.ly for  y .
*/Cone & Ysrc(const std::string &ysrc ) {
    _cone.insert({"ysrc", ysrc});
    return *this;
}


/**
Sets the z coordinates of the vector field and of the displayed cones.
*/Cone & Z(const std::vector<double> &z ) {
    _cone.insert({"z", z});
    return *this;
}


/**
Sets the z coordinates of the vector field and of the displayed cones.
*/Cone & Z(const std::vector<std::string> &z ) {
    _cone.insert({"z", z});
    return *this;
}


/**
Sets the z coordinates of the vector field and of the displayed cones.
*/Cone & Z(const json11::Json &z ) {
    _cone.insert({"z", z});
    return *this;
}


/**
Sets the source reference on plot.ly for  z .
*/Cone & Zsrc(const std::string &zsrc ) {
    _cone.insert({"zsrc", zsrc});
    return *this;
}


 

            virtual json11::Json to_json() const {
                return _cone;
            }

        private:
            
            json11::Json::object _cone;

        };
    }
}


