/**
 * @file Histogram.h
This file has been autogenerated from the official plotly.js source.

If you like Plotly, please support them: L<https://plot.ly/> 
Open source announcement: L<https://plot.ly/javascript/open-source-announcement/>

Full reference: L<https://plot.ly/javascript/reference/#histogram>

This is an unofficial Plotly Perl module. Currently I'm not affiliated in any way with Plotly. 
But I think plotly.js is a great library and I want to use it with C++.
 *
 */
#include <vector>
#include <string>
#include <sstream>

#include <json11.hpp>

#include "../BaseTrace.h"

#include "CppPlotly/Trace/Histogram/Cumulative.h"
#include "CppPlotly/Trace/Histogram/Error_x.h"
#include "CppPlotly/Trace/Histogram/Error_y.h"
#include "CppPlotly/Trace/Histogram/Hoverlabel.h"
#include "CppPlotly/Trace/Histogram/Marker.h"
#include "CppPlotly/Trace/Histogram/Selected.h"
#include "CppPlotly/Trace/Histogram/Stream.h"
#include "CppPlotly/Trace/Histogram/Transform.h"
#include "CppPlotly/Trace/Histogram/Unselected.h"
#include "CppPlotly/Trace/Histogram/Xbins.h"
#include "CppPlotly/Trace/Histogram/Ybins.h"


namespace CppPlotly {
    namespace Trace {
        /**
         * The sample data from which statistics are computed is set in `x` for vertically spanning histograms and in `y` for horizontally spanning histograms. Binning options are set `xbins` and `ybins` respectively if no aggregation data is provided.
         *
         *
         */ 
        class Histogram : public virtual CppPlotly::BaseTrace {
        public:
            Histogram() :
                            _histogram({
                                {"type", "histogram"},
                                    })
                        {}

           /**
Set several traces linked to the same position axis or matching axes to the same alignmentgroup. This controls whether bars compute their positional range dependently or independently.
*/Histogram & Alignmentgroup(const std::string &alignmentgroup ) {
    _histogram.insert({"alignmentgroup", alignmentgroup});
    return *this;
}


/**
Obsolete: since v1.42 each bin attribute is auto-determined separately and `autobinx` is not needed. However, we accept `autobinx: true` or `false` and will update `xbins` accordingly before deleting `autobinx` from the trace.
*/Histogram & Autobinx(const bool &autobinx ) {
    _histogram.insert({"autobinx", autobinx});
    return *this;
}


/**
Obsolete: since v1.42 each bin attribute is auto-determined separately and `autobiny` is not needed. However, we accept `autobiny: true` or `false` and will update `ybins` accordingly before deleting `autobiny` from the trace.
*/Histogram & Autobiny(const bool &autobiny ) {
    _histogram.insert({"autobiny", autobiny});
    return *this;
}


Histogram & Cumulative(const CppPlotly::Trace::histogram::Cumulative &cumulative ) {
    _histogram.insert({"cumulative", cumulative});
    return *this;
}


/**
Assigns extra data each datum. This may be useful when listening to hover, click and selection events. Note that, *scatter* traces also appends customdata items in the markers DOM elements
*/Histogram & Customdata(const std::vector<double> &customdata ) {
    _histogram.insert({"customdata", customdata});
    return *this;
}


/**
Assigns extra data each datum. This may be useful when listening to hover, click and selection events. Note that, *scatter* traces also appends customdata items in the markers DOM elements
*/Histogram & Customdata(const std::vector<std::string> &customdata ) {
    _histogram.insert({"customdata", customdata});
    return *this;
}


/**
Assigns extra data each datum. This may be useful when listening to hover, click and selection events. Note that, *scatter* traces also appends customdata items in the markers DOM elements
*/Histogram & Customdata(const json11::Json &customdata ) {
    _histogram.insert({"customdata", customdata});
    return *this;
}


/**
Sets the source reference on plot.ly for  customdata .
*/Histogram & Customdatasrc(const std::string &customdatasrc ) {
    _histogram.insert({"customdatasrc", customdatasrc});
    return *this;
}


Histogram & Error_x(const CppPlotly::Trace::histogram::Error_x &error_x ) {
    _histogram.insert({"error_x", error_x});
    return *this;
}


Histogram & Error_y(const CppPlotly::Trace::histogram::Error_y &error_y ) {
    _histogram.insert({"error_y", error_y});
    return *this;
}


/**
Specifies the binning function used for this histogram trace. If *count*, the histogram values are computed by counting the number of values lying inside each bin. If *sum*, *avg*, *min*, *max*, the histogram values are computed using the sum, the average, the minimum or the maximum of the values lying inside each bin respectively.
*/Histogram & Histfunc(const json11::Json &histfunc ) {
    _histogram.insert({"histfunc", histfunc});
    return *this;
}


/**
Specifies the type of normalization used for this histogram trace. If **, the span of each bar corresponds to the number of occurrences (i.e. the number of data points lying inside the bins). If *percent* / *probability*, the span of each bar corresponds to the percentage / fraction of occurrences with respect to the total number of sample points (here, the sum of all bin HEIGHTS equals 100% / 1). If *density*, the span of each bar corresponds to the number of occurrences in a bin divided by the size of the bin interval (here, the sum of all bin AREAS equals the total number of sample points). If *probability density*, the area of each bar corresponds to the probability that an event will fall into the corresponding bin (here, the sum of all bin AREAS equals 1).
*/Histogram & Histnorm(const json11::Json &histnorm ) {
    _histogram.insert({"histnorm", histnorm});
    return *this;
}


/**
Determines which trace information appear on hover. If `none` or `skip` are set, no information is displayed upon hovering. But, if `none` is set, click and hover events are still fired.
*/Histogram & Hoverinfo(const json11::Json &hoverinfo ) {
    _histogram.insert({"hoverinfo", hoverinfo});
    return *this;
}


/**
Sets the source reference on plot.ly for  hoverinfo .
*/Histogram & Hoverinfosrc(const std::string &hoverinfosrc ) {
    _histogram.insert({"hoverinfosrc", hoverinfosrc});
    return *this;
}


Histogram & Hoverlabel(const CppPlotly::Trace::histogram::Hoverlabel &hoverlabel ) {
    _histogram.insert({"hoverlabel", hoverlabel});
    return *this;
}


/**
Template string used for rendering the information that appear on hover box. Note that this will override `hoverinfo`. Variables are inserted using %{variable}, for example "y: %{y}". Numbers are formatted using d3-format's syntax %{variable:d3-format}, for example "Price: %{y:$.2f}". See https://github.com/d3/d3-format/blob/master/README.md#locale_format for details on the formatting syntax. The variables available in `hovertemplate` are the ones emitted as event data described at this link https://plot.ly/javascript/plotlyjs-events/#event-data. Additionally, every attributes that can be specified per-point (the ones that are `arrayOk: true`) are available. variable `binNumber` Anything contained in tag `<extra>` is displayed in the secondary box, for example "<extra>{fullData.name}</extra>". To hide the secondary box completely, use an empty tag `<extra></extra>`.
*/Histogram & Hovertemplate(const json11::Json &hovertemplate ) {
    _histogram.insert({"hovertemplate", hovertemplate});
    return *this;
}


/**
Sets the source reference on plot.ly for  hovertemplate .
*/Histogram & Hovertemplatesrc(const std::string &hovertemplatesrc ) {
    _histogram.insert({"hovertemplatesrc", hovertemplatesrc});
    return *this;
}


/**
Same as `text`.
*/Histogram & Hovertext(const json11::Json &hovertext ) {
    _histogram.insert({"hovertext", hovertext});
    return *this;
}


/**
Sets the source reference on plot.ly for  hovertext .
*/Histogram & Hovertextsrc(const std::string &hovertextsrc ) {
    _histogram.insert({"hovertextsrc", hovertextsrc});
    return *this;
}


/**
Assigns id labels to each datum. These ids for object constancy of data points during animation. Should be an array of strings, not numbers or any other type.
*/Histogram & Ids(const std::vector<double> &ids ) {
    _histogram.insert({"ids", ids});
    return *this;
}


/**
Assigns id labels to each datum. These ids for object constancy of data points during animation. Should be an array of strings, not numbers or any other type.
*/Histogram & Ids(const std::vector<std::string> &ids ) {
    _histogram.insert({"ids", ids});
    return *this;
}


/**
Assigns id labels to each datum. These ids for object constancy of data points during animation. Should be an array of strings, not numbers or any other type.
*/Histogram & Ids(const json11::Json &ids ) {
    _histogram.insert({"ids", ids});
    return *this;
}


/**
Sets the source reference on plot.ly for  ids .
*/Histogram & Idssrc(const std::string &idssrc ) {
    _histogram.insert({"idssrc", idssrc});
    return *this;
}


/**
Sets the legend group for this trace. Traces part of the same legend group hide/show at the same time when toggling legend items.
*/Histogram & Legendgroup(const std::string &legendgroup ) {
    _histogram.insert({"legendgroup", legendgroup});
    return *this;
}


Histogram & Marker(const CppPlotly::Trace::histogram::Marker &marker ) {
    _histogram.insert({"marker", marker});
    return *this;
}


/**
Sets the trace name. The trace name appear as the legend item and on hover.
*/Histogram & Name(const std::string &name ) {
    _histogram.insert({"name", name});
    return *this;
}


/**
Specifies the maximum number of desired bins. This value will be used in an algorithm that will decide the optimal bin size such that the histogram best visualizes the distribution of the data. Ignored if `xbins.size` is provided.
*/Histogram & Nbinsx(const int &nbinsx ) {
    _histogram.insert({"nbinsx", nbinsx});
    return *this;
}


/**
Specifies the maximum number of desired bins. This value will be used in an algorithm that will decide the optimal bin size such that the histogram best visualizes the distribution of the data. Ignored if `ybins.size` is provided.
*/Histogram & Nbinsy(const int &nbinsy ) {
    _histogram.insert({"nbinsy", nbinsy});
    return *this;
}


/**
Set several traces linked to the same position axis or matching axes to the same offsetgroup where bars of the same position coordinate will line up.
*/Histogram & Offsetgroup(const std::string &offsetgroup ) {
    _histogram.insert({"offsetgroup", offsetgroup});
    return *this;
}


/**
Sets the opacity of the trace.
*/Histogram & Opacity(const double &opacity ) {
    _histogram.insert({"opacity", opacity});
    return *this;
}


/**
Sets the orientation of the bars. With *v* (*h*), the value of the each bar spans along the vertical (horizontal).
*/Histogram & Orientation(const json11::Json &orientation ) {
    _histogram.insert({"orientation", orientation});
    return *this;
}


Histogram & Selected(const CppPlotly::Trace::histogram::Selected &selected ) {
    _histogram.insert({"selected", selected});
    return *this;
}


/**
Array containing integer indices of selected points. Has an effect only for traces that support selections. Note that an empty array means an empty selection where the `unselected` are turned on for all points, whereas, any other non-array values means no selection all where the `selected` and `unselected` styles have no effect.
*/Histogram & Selectedpoints(const json11::Json &selectedpoints ) {
    _histogram.insert({"selectedpoints", selectedpoints});
    return *this;
}


/**
Determines whether or not an item corresponding to this trace is shown in the legend.
*/Histogram & Showlegend(const bool &showlegend ) {
    _histogram.insert({"showlegend", showlegend});
    return *this;
}


Histogram & Stream(const CppPlotly::Trace::histogram::Stream &stream ) {
    _histogram.insert({"stream", stream});
    return *this;
}


/**
Sets hover text elements associated with each bar. If a single string, the same string appears over all bars. If an array of string, the items are mapped in order to the this trace's coordinates.
*/Histogram & Text(const json11::Json &text ) {
    _histogram.insert({"text", text});
    return *this;
}


/**
Sets the source reference on plot.ly for  text .
*/Histogram & Textsrc(const std::string &textsrc ) {
    _histogram.insert({"textsrc", textsrc});
    return *this;
}


Histogram & Transforms(const std::vector<CppPlotly::Trace::histogram::Transform> &transforms ) {
    _histogram.insert({"transforms", transforms});
    return *this;
}


/**
Assign an id to this trace, Use this to provide object constancy between traces during animations and transitions.
*/Histogram & Uid(const std::string &uid ) {
    _histogram.insert({"uid", uid});
    return *this;
}


/**
Controls persistence of some user-driven changes to the trace: `constraintrange` in `parcoords` traces, as well as some `editable: true` modifications such as `name` and `colorbar.title`. Defaults to `layout.uirevision`. Note that other user-driven trace attribute changes are controlled by `layout` attributes: `trace.visible` is controlled by `layout.legend.uirevision`, `selectedpoints` is controlled by `layout.selectionrevision`, and `colorbar.(x|y)` (accessible with `config: {editable: true}`) is controlled by `layout.editrevision`. Trace changes are tracked by `uid`, which only falls back on trace index if no `uid` is provided. So if your app can add/remove traces before the end of the `data` array, such that the same trace has a different index, you can still preserve user-driven changes if you give each trace a `uid` that stays with it as it moves.
*/Histogram & Uirevision(const json11::Json &uirevision ) {
    _histogram.insert({"uirevision", uirevision});
    return *this;
}


Histogram & Unselected(const CppPlotly::Trace::histogram::Unselected &unselected ) {
    _histogram.insert({"unselected", unselected});
    return *this;
}


/**
Determines whether or not this trace is visible. If *legendonly*, the trace is not drawn, but can appear as a legend item (provided that the legend itself is visible).
*/Histogram & Visible(const json11::Json &visible ) {
    _histogram.insert({"visible", visible});
    return *this;
}


/**
Sets the sample data to be binned on the x axis.
*/Histogram & X(const std::vector<double> &x ) {
    _histogram.insert({"x", x});
    return *this;
}


/**
Sets the sample data to be binned on the x axis.
*/Histogram & X(const std::vector<std::string> &x ) {
    _histogram.insert({"x", x});
    return *this;
}


/**
Sets the sample data to be binned on the x axis.
*/Histogram & X(const json11::Json &x ) {
    _histogram.insert({"x", x});
    return *this;
}


/**
Sets a reference between this trace's x coordinates and a 2D cartesian x axis. If *x* (the default value), the x coordinates refer to `layout.xaxis`. If *x2*, the x coordinates refer to `layout.xaxis2`, and so on.
*/Histogram & Xaxis(const json11::Json &xaxis ) {
    _histogram.insert({"xaxis", xaxis});
    return *this;
}


Histogram & Xbins(const CppPlotly::Trace::histogram::Xbins &xbins ) {
    _histogram.insert({"xbins", xbins});
    return *this;
}


/**
Sets the calendar system to use with `x` date data.
*/Histogram & Xcalendar(const json11::Json &xcalendar ) {
    _histogram.insert({"xcalendar", xcalendar});
    return *this;
}


/**
Sets the source reference on plot.ly for  x .
*/Histogram & Xsrc(const std::string &xsrc ) {
    _histogram.insert({"xsrc", xsrc});
    return *this;
}


/**
Sets the sample data to be binned on the y axis.
*/Histogram & Y(const std::vector<double> &y ) {
    _histogram.insert({"y", y});
    return *this;
}


/**
Sets the sample data to be binned on the y axis.
*/Histogram & Y(const std::vector<std::string> &y ) {
    _histogram.insert({"y", y});
    return *this;
}


/**
Sets the sample data to be binned on the y axis.
*/Histogram & Y(const json11::Json &y ) {
    _histogram.insert({"y", y});
    return *this;
}


/**
Sets a reference between this trace's y coordinates and a 2D cartesian y axis. If *y* (the default value), the y coordinates refer to `layout.yaxis`. If *y2*, the y coordinates refer to `layout.yaxis2`, and so on.
*/Histogram & Yaxis(const json11::Json &yaxis ) {
    _histogram.insert({"yaxis", yaxis});
    return *this;
}


Histogram & Ybins(const CppPlotly::Trace::histogram::Ybins &ybins ) {
    _histogram.insert({"ybins", ybins});
    return *this;
}


/**
Sets the calendar system to use with `y` date data.
*/Histogram & Ycalendar(const json11::Json &ycalendar ) {
    _histogram.insert({"ycalendar", ycalendar});
    return *this;
}


/**
Sets the source reference on plot.ly for  y .
*/Histogram & Ysrc(const std::string &ysrc ) {
    _histogram.insert({"ysrc", ysrc});
    return *this;
}


 

            virtual json11::Json to_json() const {
                return _histogram;
            }

        private:
            
            json11::Json::object _histogram;

        };
    }
}


