/**
 * @file Header.h
This attribute is part of the possible options for the trace table.

This file has been autogenerated from the official plotly.js source.

If you like Plotly, please support them: L<https://plot.ly/> 
Open source announcement: L<https://plot.ly/javascript/open-source-announcement/>

Full reference: L<https://plot.ly/javascript/reference/#table>

DISCLAIMER

This is an unofficial Plotly Perl module. Currently I'm not affiliated in any way with Plotly. 
But I think plotly.js is a great library and I want to use it with C++.

*/

#include <vector>
#include <string>
#include <sstream>

#include <json11.hpp>

#include "CppPlotly/Trace/Table/Header/Fill.h"
#include "CppPlotly/Trace/Table/Header/Font.h"
#include "CppPlotly/Trace/Table/Header/Line.h"



namespace CppPlotly {
    namespace Trace {
    namespace table {

    /**
     * 
     *
     *
     */
    class Header {
    public:

    /**
Sets the horizontal alignment of the `text` within the box. Has an effect only if `text` spans more two or more lines (i.e. `text` contains one or more <br> HTML tags) or if an explicit width is set to override the text width.
*/Header & Align(const json11::Json &align ) {
    _header.insert({"align", align});
    return *this;
}


/**
Sets the source reference on plot.ly for  align .
*/Header & Alignsrc(const std::string &alignsrc ) {
    _header.insert({"alignsrc", alignsrc});
    return *this;
}


Header & Fill(const CppPlotly::Trace::table::header::Fill &fill ) {
    _header.insert({"fill", fill});
    return *this;
}


Header & Font(const CppPlotly::Trace::table::header::Font &font ) {
    _header.insert({"font", font});
    return *this;
}


/**
Sets the cell value formatting rule using d3 formatting mini-language which is similar to those of Python. See https://github.com/d3/d3-format/blob/master/README.md#locale_format
*/Header & Format(const std::vector<double> &format ) {
    _header.insert({"format", format});
    return *this;
}


/**
Sets the cell value formatting rule using d3 formatting mini-language which is similar to those of Python. See https://github.com/d3/d3-format/blob/master/README.md#locale_format
*/Header & Format(const std::vector<std::string> &format ) {
    _header.insert({"format", format});
    return *this;
}


/**
Sets the cell value formatting rule using d3 formatting mini-language which is similar to those of Python. See https://github.com/d3/d3-format/blob/master/README.md#locale_format
*/Header & Format(const json11::Json &format ) {
    _header.insert({"format", format});
    return *this;
}


/**
Sets the source reference on plot.ly for  format .
*/Header & Formatsrc(const std::string &formatsrc ) {
    _header.insert({"formatsrc", formatsrc});
    return *this;
}


/**
The height of cells.
*/Header & Height(const double &height ) {
    _header.insert({"height", height});
    return *this;
}


Header & Line(const CppPlotly::Trace::table::header::Line &line ) {
    _header.insert({"line", line});
    return *this;
}


/**
Prefix for cell values.
*/Header & Prefix(const json11::Json &prefix ) {
    _header.insert({"prefix", prefix});
    return *this;
}


/**
Sets the source reference on plot.ly for  prefix .
*/Header & Prefixsrc(const std::string &prefixsrc ) {
    _header.insert({"prefixsrc", prefixsrc});
    return *this;
}


/**
Suffix for cell values.
*/Header & Suffix(const json11::Json &suffix ) {
    _header.insert({"suffix", suffix});
    return *this;
}


/**
Sets the source reference on plot.ly for  suffix .
*/Header & Suffixsrc(const std::string &suffixsrc ) {
    _header.insert({"suffixsrc", suffixsrc});
    return *this;
}


/**
Header cell values. `values[m][n]` represents the value of the `n`th point in column `m`, therefore the `values[m]` vector length for all columns must be the same (longer vectors will be truncated). Each value must be a finite number or a string.
*/Header & Values(const std::vector<double> &values ) {
    _header.insert({"values", values});
    return *this;
}


/**
Header cell values. `values[m][n]` represents the value of the `n`th point in column `m`, therefore the `values[m]` vector length for all columns must be the same (longer vectors will be truncated). Each value must be a finite number or a string.
*/Header & Values(const std::vector<std::string> &values ) {
    _header.insert({"values", values});
    return *this;
}


/**
Header cell values. `values[m][n]` represents the value of the `n`th point in column `m`, therefore the `values[m]` vector length for all columns must be the same (longer vectors will be truncated). Each value must be a finite number or a string.
*/Header & Values(const json11::Json &values ) {
    _header.insert({"values", values});
    return *this;
}


/**
Sets the source reference on plot.ly for  values .
*/Header & Valuessrc(const std::string &valuessrc ) {
    _header.insert({"valuessrc", valuessrc});
    return *this;
}




    json11::Json to_json() const {
        return _header;
    }

    private:

    json11::Json::object _header;

    };

    }

    }
}
